<?php

namespace Drupal\content_reporting\Plugin\QueueWorker;

use Drupal\Core\Batch\BatchBuilder;
use Drupal\Core\Database\Connection;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\QueueWorkerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Processes content interaction tracking tasks.
 *
 * @QueueWorker(
 *   id = "content_reporting_interactions_queue",
 *   title = @Translation("Content Reporting Interaction Worker"),
 *   cron = {"time" = 60}
 * )
 */
class ContentReportingInteractionWorker extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  public function __construct(array $configuration, $plugin_id, $plugin_definition, Connection $database) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->database = $database;
  }

  /**
   * Processes queued items in a batch.
   */
  public function run() {
    $batch = new BatchBuilder();
    $batch->setTitle(t('Processing tracking...'));

    // Claim multiple items from the queue.
    $queue = \Drupal::queue('content_reporting_track_queue');
    $items = [];
    // Adjust the batch size as needed.
    for ($i = 0; $i < 10; $i++) {
      $item = $queue->claimItem();
      if ($item) {
        $items[] = $item;
      }
      else {
        // Stop if there are no more items.
        break;
      }
    }

    // Add each item to the batch for processing.
    foreach ($items as $item) {
      // Pass data to processItem.
      $batch->addOperation([$this, 'processItem'], [$item->data]);
      // Remove the item from the queue.
      $queue->deleteItem($item);
    }

    // Set the finish callback for the batch.
    $batch->setFinishCallback('content_reporting_finished');
    batch_set($batch);
  }

  /**
   * Static construct.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('database')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    // Save the interaction in the database.
    $this->database->insert('content_reporting_interactions')
      ->fields([
        'nid' => $data['nid'],
        'uid' => $data['uid'],
        'interaction_type' => $data['interaction_type'],
        'element' => $data['element'],
        'duration' => $data['duration'],
        'timestamp' => $data['timestamp'],
      ])
      ->execute();
  }

}
